/*
GPS function file for use with the header file. 
3/22/09
By TJ Johnson

This program parses the avliable NEMA string data from a GPS unit and displays 
it to the terminal at a baud of 57600. 

The terminal display is optimized for the PST terminal avliable from parallax's 
website.

For use with the parallax GPS unit, the /raw pin should be connected to ground.

See end of file for MIT License.

*/



#include "gpsParserDefinesHeader.h"



 static int GPSPIN, cogId;
 static char time[8],valid, lat[12], lon[12],speed[8], heading[8], date[8],sats[2],
	  horisontalAcc[5], alt[8];
 static long stack[50];


//Function Implementation
int gpsStart(int Pin)
{
FdSerial_start(Pin, -1, 0,4800);  //Start FdSerial
cogId = cognew(getData, &stack[49]); // Start a cog to run the parser
return cogId; // Return the id of the cog 
}

void gpsStop(void)
{
int id ;

id = cogId; // Move to local variable so it can process it
FdSerial_stop(); // Stop FD serial
asm("cogstop %id"); // Stop the parser cog
}
static void getData()
{
  int i, fieldNum;
  char rxByte;
  char nemaCode[5];
    while(1) // start the infinate string getting loop
    {
   	  while(FdSerial_rx() != '$'); // $ start of any nema string
	
	  for(i = 0; i<5; i++)        // Get the nema code from the start of the string
	    {	  	 	  			  // Nema code is 1st 5 letters after $
	    nemaCode[i] = FdSerial_rx(); 
	    }
	  if(memcmp("GPRMC", &nemaCode, 5) == 0) // check the string code - GPRMC
	   {
	   decodeGPRMC();// Call the GPRMC decode function	  
	   }
	  else if (memcmp("GPGGA", &nemaCode, 5) == 0)  // check the string code- GPGGA
	   {
	   decodeGPGGA();  // Call the GPGGA decode function
	   }   
    }

  }

static void decodeGPRMC(void)
{
int i, fieldNum;
char rxByte;


       i = 0;			   // zero our string index counter
	   fieldNum = 1;
	   rxByte = FdSerial_rx();	// give RxByte a value, should be the 1st ','
			   			  
	   while(rxByte != '*')       //Repeat until end of nema string defined by *
	     {
	   
	     rxByte = FdSerial_rx(); // Get the 1st serial byte
	     if (rxByte == ',') // if its a field sperator char
	   	    {
	   	    fieldNum++; // Increment the field data is going into 
			
			// if its 4(lat) allow to appened e or w char or 6(long), dont reset
			// string counter i.
		    if ((fieldNum != 4) && (fieldNum != 6)) 
		      {
	   	      i = 0;
		      }
		    
			//rxByte = FdSerial_rx(); // Get a new byte because we got a comma before
	   	    }
	  else
	  	  {
			 
	     	 switch (fieldNum) // Depending on what field we are in appened the string.
	         		{
	   	    		case 1:
		       	    	 time[i] = rxByte; // Time field
			   			 break;
	                case 2:
		   	                    // Non usefull field
					     break;
		    		case 3:
		       			 lat[i] = rxByte; // Lat field
			   			 break;
		    		case 4:
		       			 lat[i] = rxByte; // Lat E or W char
			   			 break;
		            case 5:
		   	   	   		lon[i] = rxByte; // Long field
			   			break;
		            case 6:
		       	   		lon[i] = rxByte;	// Long N or S 	  	 	 
			   			break;
		    	   case 7:
		       	   		speed[i] = rxByte; // Speed Field
			   			break;
		    	   case 8:
		       	   		heading[i] = rxByte; // Heading Field
			   			break;
		    	   case 9:
		       	   		date[i] = rxByte; // Date Stamp
			   			break;
			 
		     }	 	 	 	 
		
	     i++; // Increment string counter
	   }
	 }

}  

static void decodeGPGGA(void)
{

int i, fieldNum;
char rxByte;


       i = 0;			   // zero our string index counter
	   fieldNum = 1;
	   rxByte = FdSerial_rx();	// give RxByte a value, should be the 1st ','
			   			  
	   while(rxByte != '*')       //Repeat until end of nema string
	     {
	     
		 
	     rxByte = FdSerial_rx(); // get a new serial byte.
	     if (rxByte == ',') // if its comma field seperator
	   	    {
	   	    fieldNum++; // invrement field number
			i = 0; // reset string counter
		  
			//rxByte = FdSerial_rx(); // get a new byte
	   	    }
	   	 else
		 {
	    
			  switch (fieldNum)
	          		 {
					 case 6:
			  		 	  valid = rxByte; // Valid Field
			  			  break;			
		    		 case 7:
		       		 	  sats[i] = rxByte; // Number of Sats in view
			   			  break;
		    	     case 8:
		       		 	  horisontalAcc[i] = rxByte; // Horisontal Accuracy
			   			  break;
		    	     case 9:
		       		 	  alt[i] = rxByte; // Altitude field
			   			  break;
			 
		    }	 	 	 	 
		
	     i++; // Increment string counter
	   
	     }
		 
     }
}

int getGpsValid(void)
{
 return valid-'0'; // Convert the ascii valid char to a int
}  

char* getGpsTime(void)
  {
  return time; // Return the ponter to the string time.
  }
  
  // TIME LAYOUT HHMMDD
int getGpsTimeHours(void)
  {
  char array[3];
  array[0]=time[0]; // Place 1st digit in array[0]
  array[1]=time[1];// Place second digit in 2
  array[2]=0; // add 0 terminator
  
  return atoi(array); // Return ascii to int of the array
  }  

  int getGpsTimeMin(void)
  {
  char array[3]; 
  array[0]=time[2]; // Move 3rd digit
  array[1]=time[3]; // 4th digit
  array[2]=0; // 0 terminator
   
  return atoi(array); // Return ascii to int of array
  }  
  
  int getGpsTimeSec(void)
  {
  
  char array[3];
  array[0]=time[4];// move 5th digit
  array[1]=time[5];// move 6th digit
  array[2]=0; // 0 terminator
  
  return atoi(array); // convert to int
  }  

//Date Format DDMMYY  
  
char* getGpsDate(void)
{
return (date); // Return pointer to the date array
}

  
int getGpsDay(void)
  {
  char array[3];
  array[0]=date[0]; //Move digit 1
  array[1]=date[1]; // move digit 2
  array[2]=0;   // 0 Terminator
  
  return atoi(array); // Return as int
  
  }
 
 int getGpsMonth(void)
  {
  char array[3]; 
  array[0]=date[2]; // Move digit 1
  array[1]=date[3];// move digit 2
  array[2]=0; // 0 terminator
  
  return atoi(array); // Return as int
  
  } 
   
int getGpsYear(void)
  {
  char array[3];
  array[0]=date[4]; // Move digit 1
  array[1]=date[5]; // move digit 2
  array[2]=0; // 0 termiator
  
  return atoi(array); // Return as int
  
  }
char* getGpsLat(void)
  {
  return lat; // Return pointer to lat string
  }  
      
int getGpsLatDeg(void)
  {
  return getDeg(lat); // Return the value of of lat degs as an int
  }

int getGpsLatMin(void)
  {
  return getMin(lat); //Return the value of lat min as an int
  }

int getGpsLatSec(void)
  {
  return getDecSec(lat); // Return lat seconds as an int
  }
    
float getGpsLatDecMin(void)
  {
  // Decimal Minute Example 52.4544'
  return getDecMin(lat); // Return the decimal minutes
  }
  
char* getGpsLong(void)
  {
  return lon; // Return pointer to Long String
  }

int getGpsLongDeg(void)
  {
  return getDeg(lon); // Return Long Degrees
  }
  
int getGpsLongMin(void)  
  {
  return getMin(lon); // Return Long Min
  }
  
int getGpsLongSec(void)
  {
  return getDecSec(lon); //Return Long Seconds
  }  
  
float getGpsLongDecMin(void)
  {
  // Dec min example 52.5466
  return getDecMin(lon); // Return long dec minutes
  }  
        
int getGpsAlt(void)
  {
  return atoi(alt); // Return alt as an interger ( unit Meters)
  }

int getGpsAltTenths(void)
  {
  return (alt[6]-'0'); // Return the 10ths of the altitude
  }     
 
float getGpsAltFloat(void)
  {
  return atof(alt); // Return altitude as a float
  }
    
int getGpsHeadingDeg(void)
  {
  return atoi(heading); // Return interger value of heading
  }

  int getGpsHeadingTenths(void)
  {
   
  return heading[4]-'0'; // return the tenths of the heading
  
  }

  
float getGpsHeadingFloat(void)
{
return atof(heading); // Return heading as a float
}

int getGpsSats(void)
  {
  return atoi(sats); // Return current number of satilites in view
  }
  
    



 int getDeg(char *array)
{
  char decimalLoc, length;
  char deg[3];
  int degInt;
 
  decimalLoc = charLoc(array, '.'); // Find the location of the Dec in string
  length = strlen(array); // Find the length of the string
		   switch(decimalLoc) 
		   	  {
			  // string always places deg(1 - 3 digits) min(2 digit) decimal
			   case 3: // if 1 digit deg, 
			   		deg[0] = 0;
					deg[1] = 0;
					deg[2] = array[0];
			  
			   case 4: // if two digit degree
			   		deg[0] = '0'; 
					deg[1] = array[0];
					deg[2] = array[1];
			   		break;
			   case 5: // if 3 digit degree
			   		deg[0] = array[0];
					deg[1] = array[1];
					deg[2] = array[2];
					break;
					
			  } 
			  
		degInt = atoi(deg); // convert to interger value
		if (array[length-1] == 'W' || array[length-1] == 'S') // if w or s make negitibe
		{
		degInt *= -1;
		}	  
  return degInt; // return the deg interger.
}

 int getMin(char *array)
{
  int decimalLoc;
  char min[3];

  decimalLoc = charLoc(array, '.'); // find the decimal
  
  // copy two chars in front of the decimal
  min[2] = 0; // make last digit 0 term for string.
  min[1] = array[decimalLoc-1];
  min[0] = array[decimalLoc-2];

  return atoi(min); // return number converted to string
}

 float getDecMin(char *array)
{
  int length, decimalLoc, index;
  char decMin[10];
  
   //Find the location of the decimal point in the  
  decimalLoc = charLoc(array, '.');
  
  // Get the length of the total string
  length = strlen(array)-1; //Subtract 1 so we dont copy the NS or EW id char
  
  index = decimalLoc - 2; // Move our start index back 2 places
  // Starting at 2 digits in front of the decimal place 
  for(decimalLoc -= 2; decimalLoc <= length; decimalLoc++) 
  {
   //Copy the correct digits from the array
   decMin[decimalLoc - index] = array[decimalLoc];  
  }
  
  // return the floating point number of DecMin converting from string to float
  return atof(decMin);

}

 int getDecSec(char *array)
{
  int decimalLoc, length, index;
  char decMin[10];
  
  //Find the location of the decimal
  decimalLoc = charLoc(array, '.');
  
  //Get the length of the string 
  length = strlen(array);
  
  // Set our index point just behind the decimal
  index = decimalLoc+1;
  
  // Starting at 2 digits in front of the decimal place copy 
  //to the new array one digit at a time
  for(decimalLoc ++; decimalLoc <= length; decimalLoc++)
  {
   decMin[decimalLoc - index] = array[decimalLoc];  
  }
  
  // return the interger value devided by 60.
  return atoi(decMin)/60;

}

static int charLoc(char *array, char searchChar)
{
int index;

// search for the given character in a c_str array until we find the end or the 1st char
for(index = 0; (array[index] !=  0) && (array[index] != searchChar); index++);
// return the 1st occurance of the char, or last element number.
return index;
}

  /*
+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
*/